<?php
/*
Plugin Name: FluentCRM Dashboard Widget
Plugin URI:  https://wpscriptly.com/
Description: Adds a dashboard widget for FluentCRM stats.
Version:     1.5.8
Author:      SB
Author URI:  https://wpscriptly.com/
License:     GPL2
*/

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// Hook to add dashboard widget
add_action('wp_dashboard_setup', 'fluentcrm_add_dashboard_widgets');

// Hook to enqueue scripts
add_action('admin_enqueue_scripts', 'fluentcrm_enqueue_scripts');

// Enqueue the Dashicons script
add_action('wp_enqueue_scripts', 'load_dashicons_front_end');

function fluentcrm_add_dashboard_widgets() {
    wp_add_dashboard_widget(
        'fluentcrm_stats_dashboard_widget', // Widget slug
        'FluentCRM Stats', // Title
        'fluentcrm_dashboard_widget_display' // Display function
    );
}

function fluentcrm_dashboard_widget_display() {
    $campaigns = fluentcrm_get_campaigns();
    if (!$campaigns) {
        echo '<p>Could not retrieve FluentCRM campaigns.</p>';
        return;
    }

    echo '<select id="fluentcrm-newsletter-dropdown">';
    foreach ($campaigns as $campaign) {
        echo '<option value="' . esc_attr($campaign->id) . '">' . esc_html($campaign->title) . '</option>';
    }
    echo '</select>';

    echo '<div id="fluentcrm-stats-container"></div>';
    echo '<div id="fluentcrm-chart-container"><canvas id="fluentcrm-chart"></canvas></div>';

    echo '<div class="fluentcrm-footer">Created By: 
            <a href="https://wpscriptly.com" target="_blank">
                <img src="https://wpscriptly.com/wp-content/uploads/2024/05/wpsc-logo-256.png" alt="WpScriptly Logo"> WpScriptly
            </a>
          </div>';
}

function fluentcrm_get_campaigns() {
    if (!class_exists('FluentCrm\App\Models\Campaign')) {
        return false;
    }

    return FluentCrm\App\Models\Campaign::all();
}

function fluentcrm_get_stats() {
    if (!class_exists('FluentCrm\App\Models\Campaign') ||
        !class_exists('FluentCrm\App\Models\CampaignEmail') ||
        !class_exists('FluentCrm\App\Models\CampaignUrlMetric')) {
        return false;
    }

    $campaign_id = isset($_POST['campaign_id']) ? intval($_POST['campaign_id']) : 0;
    if (!$campaign_id) {
        return false;
    }

    // Fetch campaign data
    $campaign = FluentCrm\App\Models\Campaign::find($campaign_id);
    if (!$campaign) {
        return false;
    }

    // Fetch total emails sent for this campaign
    $total_emails_sent = FluentCrm\App\Models\CampaignEmail::where('campaign_id', $campaign_id)->count();

    // Fetch open rate for this campaign
    $total_opens = FluentCrm\App\Models\CampaignUrlMetric::where('type', 'open')->where('campaign_id', $campaign_id)->count();
    $open_rate = $total_emails_sent ? ($total_opens / $total_emails_sent) * 100 : 0;

    // Fetch click rate for this campaign
    $total_clicks = FluentCrm\App\Models\CampaignUrlMetric::where('type', 'click')->where('campaign_id', $campaign_id)->count();
    $click_rate = $total_emails_sent ? ($total_clicks / $total_emails_sent) * 100 : 0;

    // Calculate Click-to-Open Rate (CTOR)
    $ctor_rate = $total_opens ? ($total_clicks / $total_opens) * 100 : 0;

    // Format the sent date
    $sent_date = date('F j, Y', strtotime($campaign->created_at));

    return [
        'statsHTML' => generate_stats_html($campaign->title, $sent_date, $total_emails_sent, $open_rate, $click_rate, $ctor_rate, $campaign_id),
        'openRate' => $open_rate,
        'clickRate' => $click_rate,
        'ctorRate' => $ctor_rate,
        'totalEmailsSent' => $total_emails_sent
    ];
}

function generate_stats_html($title, $sent_date, $total_emails_sent, $open_rate, $click_rate, $ctor_rate, $campaign_id) {
    $stats = [
        'Title' => $title,
        'Sent Date' => $sent_date,
        'Total Emails Sent' => $total_emails_sent,
        'Open Rate' => number_format($open_rate, 2) . '%',
        'Click Rate' => number_format($click_rate, 2) . '%',
        'Click-to-Open Rate' => number_format($ctor_rate, 2) . '%'
    ];

    $html = '<ul class="fluentcrm-stats-list">';
    foreach ($stats as $key => $value) {
        $class = strtolower(str_replace(' ', '-', $key));
        $html .= '<li class="' . esc_attr($class) . ' dashicons-before dashicons-' . esc_attr(get_dashicon_class($key)) . '"><strong>' . esc_html($key) . ':&nbsp; </strong> ' . esc_html($value) . '</li>';
    }
    $html .= '</ul>';
    $html .= '<div class="fluentcrm-stats-button">';
    $html .= '<a href="' . admin_url('admin.php?page=fluentcrm-admin#/email/campaigns/' . $campaign_id . '/view') . '" target="_blank">View Campaign</a>';
    $html .= '</div>';

    return $html;
}

function get_dashicon_class($key) {
    switch ($key) {
        case 'Title':
            return 'admin-post';
        case 'Sent Date':
            return 'calendar-alt';
        case 'Total Emails Sent':
            return 'email';
        case 'Open Rate':
            return 'visibility';
        case 'Click Rate':
        case 'Click-to-Open Rate':
            return 'admin-links';
        default:
            return 'admin-generic';
    }
}

add_action('wp_ajax_fluentcrm_get_stats', 'fluentcrm_ajax_get_stats');

function fluentcrm_ajax_get_stats() {
    $stats = fluentcrm_get_stats();
    if (!$stats) {
        echo json_encode(['statsHTML' => '<p>Could not retrieve FluentCRM stats.</p>']);
        wp_die();
    }

    echo json_encode($stats);
    wp_die();
}

function fluentcrm_enqueue_scripts($hook) {
    if ('index.php' != $hook) {
        return;
    }

    wp_enqueue_script('jquery');
    wp_enqueue_script('fluentcrm-chart', plugin_dir_url(__FILE__) . 'js/fluentcrm-chart.js', ['jquery'], null, true);
    wp_enqueue_style('fluentcrm-styles', plugin_dir_url(__FILE__) . 'css/fluentcrm-styles.css');
    wp_enqueue_style('dashicons');
}

function load_dashicons_front_end() {
    wp_enqueue_style('dashicons');
}
?>
